function G = ewald_3d_array(Dx, Dy, Dz, k, dx, dy, dz)
% EWALD SUMMATION OF 3D ARRAY
%
%   function G = ewald_3d_array(Dx, Dy, Dz, k, dx, dy, dz)
%
%   [1] A. Kustepeli, ANALYSIS AND IMPLEMENTATION OF THE EWALD METHOD FOR
%       WAVEGUIDE AND CAVITY STRUCTURES, PhD Dissertation, Graduate School
%       of Clemson University, 1999.
%   [2] U. Carlberg et. al., Study of Antennas..., TEMC Nov. 2005.

% By Ulf Carlberg.

% Ewald splitting parameter from Eq. (14) in [2].
E = real(k) / 4;

% Calculate G1 (spectral term) from Eq. (2) in [2].
G = zeros(size(dx));
tmp = real(k) / pi;
[m, n, p] = ndgrid(0:ceil(tmp*Dx), 0:ceil(tmp*Dy), 0:ceil(tmp*Dz));
alpha2 = (pi*m/Dx).^2 + (pi*n/Dy).^2 + (pi*p/Dz).^2 - (k/2).^2;
f = (1+(m~=0)) .* (1+(n~=0)) .* (1+(p~=0)) .* exp(-alpha2/E^2) ./ ...
    (4*Dx*Dy*Dz*alpha2);
for ii = 1:numel(m)
    G = G + f(ii) * cos(2*pi*dx*m(ii)/Dx) .* cos(2*pi*dy*n(ii)/Dy) .* ...
        cos(2*pi*dz*p(ii)/Dz);
end

% Calculate G2 (spatial term) from Eq. (3) in [2].
[m, n, p] = ndgrid(0, 0, 0);  % enough for large cavities
for ii = 1:numel(m)
    R = hypot(dx-m(ii)*Dx, dy-n(ii)*Dy); 
    R = hypot(R          , dz-p(ii)*Dz); 
    G = G + exp( (0.5*k/E)^2 - (R*E).^2 ) ./ (8*pi*R) .* ...
        ( faddeeva(1j*R*E+0.5*k/E) + faddeeva(1j*R*E-0.5*k/E) );
end

end
