% Beamforming from MRC.
%
% Complement to Figure 3.8 in Foundations of Antennas by Per-Simon Kildal.

% By Ulf Carlberg.

% Assume y-directed incremental sources located on coordinated points
% (xpos,0,0).

% 4-ports array:
xpos = [-0.75 -0.25 0.25 0.75];

% 10-ports array:
%xpos = [-2.25 -1.75 -1.25 -0.75 -0.25 0.25 0.75 1.25 1.75 2.25];

k = 2*pi;
theta_deg = (-90:90)';
G = zeros(numel(theta_deg), numel(xpos));
for ii = 1:numel(xpos)
    G(:,ii) = exp(1j*k*xpos(ii)*sind(theta_deg));
end

% Define different cases with incoming plane waves from a certain directions
% with certain phases. (The phi angle is assumed to be zero, and the amplitude
% of all waves are assumed to be one.)
w_theta_deg = {0, 60, [0 60], [0  60], [-40 0 20 60], [-40 0  20  60], ...
    [-40   0 20  60], -70:20:70, -70:20:70};
w_phase_deg = {0,  0, [0  0], [0 180], [  0 0  0  0], [  0 0 180 180], ...
    [  0 180  0 180],   (0:7)*0, (0:7)*180};

% Go through all the cases and plot the resulting radiation pattern from MRC
% combination of the received signal.
G_MRC = zeros(numel(theta_deg), numel(w_theta_deg));
for icase = 1:numel(w_theta_deg)

    % Calculate received voltage on the two antenna ports from incoming plane
    % waves.
    th = w_theta_deg{icase};
    ph = w_phase_deg{icase};
    Q = numel(th);
    v = zeros(1, numel(xpos));
    for ii = 1:numel(xpos)
        for q = 1:Q
            v(ii) = v(ii) + G(theta_deg==th(q),ii)*exp(1j*ph(q)*pi/180)/sqrt(Q);
        end
    end

    % Calculate MRC combined radiation pattern.
    for ii = 1:numel(xpos)
        G_MRC(:,icase) = G_MRC(:,icase) + conj(v(ii))*G(:,ii);
    end
end

% Plot.
figure;
plot(theta_deg, 20*log10(abs(G_MRC(:,1))), 'k-' , 'LineWidth', 1); hold on;
plot(theta_deg, 20*log10(abs(G_MRC(:,2))), 'r--', 'LineWidth', 1);
axis([-90 90 -20 22.1]);
set(gca, 'XTick', -90:30:90);
grid on;
title(' ');
xlabel('\theta (deg)');
ylabel('Relative level (dB)');
legend('Plane wave at \theta = 0^\circ', 'Plane wave at \theta = 60^\circ', ...
    'Location', 'southwest');
if exist('savefigs', 'var') && savefigs
    save_fig('Fig3-08a.eps', 12.5*0.6, 4*1);
end

figure;
plot(theta_deg, 20*log10(abs(G_MRC(:,3))), 'k-' , 'LineWidth', 1); hold on;
plot(theta_deg, 20*log10(abs(G_MRC(:,4))), 'r--', 'LineWidth', 1);
axis([-90 90 -20 22.1]);
set(gca, 'XTick', -90:30:90);
grid on;
title('Plane waves at \theta = [0, 60]^\circ');
xlabel('\theta (deg)');
ylabel('Relative level (dB)');
legend('Phase [0, 0]^\circ', 'Phase [0, 180]^\circ', 'Location', 'southwest');
if exist('savefigs', 'var') && savefigs
    save_fig('Fig3-08b.eps', 12.5*0.6, 4*1);
end

figure;
plot(theta_deg, 20*log10(abs(G_MRC(:,5))), 'k-' , 'LineWidth', 1); hold on;
plot(theta_deg, 20*log10(abs(G_MRC(:,6))), 'r--', 'LineWidth', 1);
axis([-90 90 -20 22.1]);
set(gca, 'XTick', -90:30:90);
grid on;
title('Plane waves at \theta = [-40, 0, 20, 60]^\circ');
xlabel('\theta (deg)');
ylabel('Relative level (dB)');
legend('Phase [0, 0, 0, 0]^\circ', 'Phase [0, 0, 180, 180]^\circ', ...
    'Location', 'southwest');
if exist('savefigs', 'var') && savefigs
    save_fig('Fig3-08c.eps', 12.5*0.6, 4*1);
end

figure;
plot(theta_deg, 20*log10(abs(G_MRC(:,5))), 'k-' , 'LineWidth', 1); hold on;
plot(theta_deg, 20*log10(abs(G_MRC(:,7))), 'r--', 'LineWidth', 1);
axis([-90 90 -20 22.1]);
set(gca, 'XTick', -90:30:90);
grid on;
title('Plane waves at \theta = [-40, 0, 20, 60]^\circ');
xlabel('\theta (deg)');
ylabel('Relative level (dB)');
legend('Phase [0, 0, 0, 0]^\circ', 'Phase [0, 180, 0, 180]^\circ', ...
    'Location', 'southwest');
if exist('savefigs', 'var') && savefigs
    save_fig('Fig3-08d.eps', 12.5*0.6, 4*1);
end

figure;
plot(theta_deg, 20*log10(abs(G_MRC(:,8))), 'k-' , 'LineWidth', 1); hold on;
plot(theta_deg, 20*log10(abs(G_MRC(:,9))), 'r--', 'LineWidth', 1);
axis([-90 90 -20 22.1]);
set(gca, 'XTick', -90:30:90);
grid on;
title('Plane waves at \theta = [-70, -50, -30, -10, 10, 30, 50, 70]^\circ');
xlabel('\theta (deg)');
ylabel('Relative level (dB)');
legend('Phase [0, 0, 0, 0, 0, 0, 0, 0]^\circ', ...
    'Phase [0, 180, 0, 180, 0, 180, 0, 180]^\circ', 'Location', 'southwest');
if exist('savefigs', 'var') && savefigs
    save_fig('Fig3-08e.eps', 12.5*0.6, 4*1);
end
