% Beamforming from MRC.
%
% Complement to Figure 3.8x in Foundations of Antennas by Per-Simon Kildal.

% By Ulf Carlberg.

% Assume y-directed incremental sources located on coordinated points
% (xpos,0,0).
xpos = [-0.75 -0.25 0.25 0.75];
k = 2*pi;
theta_deg = (-90:90)';
G = zeros(numel(theta_deg), numel(xpos));
for ii = 1:numel(xpos)
    G(:,ii) = exp(1j*k*xpos(ii)*sind(theta_deg));
end

% Define different cases with incoming plane waves from a certain directions
% with certain phases. (The phi angle is assumed to be zero, and the amplitude
% of all waves are assumed to be one.)
w_theta_deg = {0, 60, [0 60], [0  60], [-40 0 20 60], [-40 0  20  60], ...
    [-40   0 20  60], -70:20:70, -70:20:70};
w_phase_deg = {0,  0, [0  0], [0 180], [  0 0  0  0], [  0 0 180 180], ...
    [  0 180  0 180],   (0:7)*0, (0:7)*180};
w_phase_deg_vector = (0:180)';

% Go through all the cases and plot the resulting radiation pattern from MRC
% combination of the received signal.
C = zeros(numel(w_phase_deg_vector), numel(w_theta_deg));
for icase = 1:numel(w_theta_deg)

    % Calculate received voltage on the two antenna ports from incoming plane
    % waves.
    th = w_theta_deg{icase};
    ph = w_phase_deg{icase};
    Q = numel(th);
    v = zeros(numel(w_phase_deg_vector), numel(xpos));
    for ii = 1:numel(xpos)
        for q = 1:Q
            if mod(ph(q),360) == 0
                ph_rad = ph(q)*pi/180;
            else
                ph_rad = w_phase_deg_vector*pi/180;
            end
            v(:,ii) = v(:,ii) + G(theta_deg==th(q),ii)*exp(1j*ph_rad)/sqrt(Q);
        end
        C(:,icase) = C(:,icase) + v(:,ii).*conj(v(:,ii));
    end
end

figure;
plot(w_phase_deg_vector, 10*log10(abs(C(:,[4 6 7 9]))), 'LineWidth', 1);
axis([0 180 -8 8]);
set(gca, 'XTick', 0:30:180);
grid on;
xlabel('\alpha (deg)');
ylabel('Capacity (dB)');
legend('\theta = [0, 60]^\circ, \Phi = [0, \alpha]', ...
    '\theta = [-40, 0, 20, 60]^\circ, \Phi = [0, 0, \alpha, \alpha]', ...
    '\theta = [-40, 0, 20, 60]^\circ, \Phi = [0, \alpha, 0, \alpha]', ...
    ['\theta = [-70, -50, -30, -10, 10, 30, 50, 70]^\circ, ' ...
    '\Phi = [0, \alpha, 0, \alpha, 0, \alpha, 0, \alpha]'], ...
    'Location', 'southwest');
if exist('savefigs', 'var') && savefigs
    save_fig('fig_03_08x.eps', 12.5*1.3, 7.5*1.3);
end
