% 3D radiation patterns of an incremental dipole and a halfwave dipole.
%
% Complement to Figure 4.8 in Foundations of Antennas by Per-Simon Kildal.

% By Ulf Carlberg.
% Edited by Madeleine Kildal Schilliger 2014-07-09

% Define theta and phi angles.
theta_deg = 0:180;
phi_deg = -180:180;
[theta, phi] = ndgrid(theta_deg*pi/180, phi_deg*pi/180);

% Far-field function (normalized) of a y-directed incremental electric current from
% Eq. (4.70).
G_id_theta = cos(theta) .* sin(phi);
G_id_phi = cos(phi);

% Far-field function (normalized) of a y-directed incremental magnetic current from
% Eq. (4.80).
G_im_theta = sin(phi);
G_im_phi = cos(theta) .* cos(phi);

% Far-field function (normalized) of a y-directed Huygens source from
% Eq. (4.81).
G_h_theta = 2*cos(theta/2).^2 .* sin(phi);
G_h_phi = 2*cos(theta/2).^2 .* cos(phi);

%% 3D radiation pattern
min_dB = -10;
max_dB = 5;

%Plot total 3D radiation pattern 
G_id_tot = sqrt(abs(G_id_theta).^2+abs(G_id_phi).^2);
P_id = trapz(phi_deg*pi/180,trapz(theta_deg*pi/180,G_id_tot.^2 .* sin(theta))); % Eq. (2.65)
D_id = G_id_tot.^2 * 4*pi/P_id;
D_id_dBi = 10*log10(D_id);
D_id_dBi_scaled = max(D_id_dBi,min_dB);

G_im_tot = sqrt(abs(G_im_theta).^2+abs(G_im_phi).^2);
P_im = trapz(phi_deg*pi/180,trapz(theta_deg*pi/180,G_im_tot.^2 .* sin(theta)));
D_im = G_im_tot.^2 * 4*pi/P_im;
D_im_dBi = 10*log10(D_im);
D_im_dBi_scaled = max(D_im_dBi,min_dB);

G_h_tot = sqrt(abs(G_h_theta).^2+abs(G_h_phi).^2); 
P_h = trapz(phi_deg*pi/180,trapz(theta_deg*pi/180,G_h_tot.^2 .* sin(theta)));
D_h = G_h_tot.^2 * 4*pi/P_h;
D_h_dBi = 10*log10(D_h);
D_h_dBi_scaled = max(D_h_dBi,min_dB);

figure;
zoomfactor = 2.0;
s1=subplot(1,3,1);
plot_3d_farfield_sub_log(theta, phi, D_id)
zoom(zoomfactor)
annotation('textbox', [0.05,0.20,0.25,0.10], 'string', 'y-pol: Incremental electric current', ...
    'HorizontalAlignment', 'center');
caxis([min_dB max_dB]);

s2=subplot(1,3,2);
plot_3d_farfield_sub_log(theta, phi, D_im)
zoom(zoomfactor)
annotation('textbox', [0.35,0.2,0.25,0.10], 'string', 'y-pol: Incremental magnetic current', ...
    'HorizontalAlignment', 'center');
caxis([min_dB max_dB]);

s3=subplot(1,3,3);
plot_3d_farfield_sub_log(theta, phi, D_h)
zoom(zoomfactor)
annotation('textbox', [0.65,0.2,0.25,0.10], 'string', 'y-pol: Huygens source', ...
    'HorizontalAlignment', 'center');
caxis([min_dB max_dB]);

s1Pos = get(s1,'position');
set(s1,'position',s1Pos-[0.07 0 0 0])
s2Pos = get(s2,'position');
set(s2,'position',s2Pos-[0.06 0 0 0])
s3Pos = get(s3,'position');
hb = colorbar('location','eastoutside');
set(s3,'position',s3Pos-[0.045 0 0 0]);
hbPos = get(hb,'position');
set(hb,'position',hbPos+[-0.01 0.21 0.0005 -0.4]);
colorTitleHandle = get(hb,'Title');
titleString = 'dB';
set(colorTitleHandle ,'String',titleString);
colormap(jet());

if exist('savefigs', 'var') && savefigs
    save_fig('Fig4-08a.eps', 14, 10, 450);
    colormap(flipud(pink((abs(min_dB)+abs(max_dB))*1)));
    save_fig('Fig4-08agray.eps', 14, 10, 450);
end