% Radiation pattern of a loop antenna.
%
% Complement to Figure 5.19 in Foundations of Antennas by Per-Simon Kildal.

% By Ulf Carlberg.

% Define dimension of loop.
circumference_lambda = 1;
a_lambda = circumference_lambda / (2*pi);
ka = 2*pi*a_lambda;

% Define angles.
theta_deg = -180:180;
phi_deg = 0:180;
[theta, phi] = ndgrid(theta_deg*pi/180, phi_deg*pi/180);

% Calculate Eq. (5.80).
J0 = besselj(0, ka*sin(theta));
J2 = besselj(2, ka*sin(theta));
Iy_x = 1j*J0 + 1j*J2.*exp(-2j*phi);
Iy_y =    J0 -    J2.*exp(-2j*phi);

% Calculate far-field.
G_theta = Iy_x .* cos(theta).*cos(phi) + Iy_y .* cos(theta).*sin(phi);
G_phi = -Iy_x .* sin(phi) + Iy_y .* cos(phi);

% Calculate co- and cross-polar fields for RHC and phi=0
G_co = (G_theta(:,phi_deg==0) + 1j*G_phi(:,phi_deg==0)) ./ sqrt(2);
G_xp = (G_theta(:,phi_deg==0) - 1j*G_phi(:,phi_deg==0)) ./ sqrt(2);

% Plot 2-D radiation patterns when phi==0.
figure;
plot(theta_deg, 20*log10(abs(G_co./max(G_co))), 'b-', ...
     theta_deg, 20*log10(abs(G_xp./max(G_co))), 'r-');
axis([-180 180 -30 0]);
grid on;
title('Loop antenna, \phi = 0');
xlabel('\theta [deg]');
ylabel('Relative level [dB]');
legend('RHC Co-pol. / LHC Cross-pol.', 'RHC Cross-pol. / LHC Co-pol.', ...
    'Location', 'southeast');

% Plot total 3-D radiation patterns.
plot_3d_farfield(theta, phi, G_theta, G_phi, 'RHC');
annotation('textbox', [0.05,0.05,0.9,0.08], 'string', 'Loop antenna', ...
    'HorizontalAlignment', 'center');
