% Radiation resistance and resonant length of a patch antenna (two-slot model).
%
% Complement to Figure 6.7 in Foundations of Antennas by Per-Simon Kildal.

% By Ulf Carlberg.

% Define dimension patch.
w = linspace(0.1, 1, 20);  % [lambda]
h = [0.01 0.05];           % [lambda]
eps_r = [1 2.54 10.2];

% Define angles and constant.
theta = (0:180)' * pi/180;
phi = (0:180) * pi/180;
[theta2D, phi2D] = ndgrid(theta, phi);
eta = 120*pi;

% Index all parameters and loop through all combinations.
[iw, ih, ie] = ndgrid(1:numel(w), 1:numel(h), 1:numel(eps_r));
G = zeros(size(iw));
l_res = zeros(size(iw));
lbls = cell(numel(h), numel(eps_r));
for ii = 1:numel(iw)

    % Calculate effective parameters.
    ww = w(iw(ii));
    hh = h(ih(ii));
    eeps_r = eps_r(ie(ii));
    F = 6 + (2*pi-6)*exp(-4*pi^2/3*(hh/ww)^(3/4));
    w_eff = 2*pi*hh / log(hh*F/ww + sqrt(1+(2*hh/ww)^2));
    eps_eff = (eeps_r+1)/2 + 0.5*(eeps_r-1)/sqrt(1+10*hh/ww);
    l_eq = 0.5 / sqrt(eps_eff);
    Delta_l = 0.412*hh*(eps_eff+0.300)/(eps_eff-0.258);
    l_res(iw(ii), ih(ii), ie(ii)) = l_eq - 2*Delta_l;

    % Plot label.
    if iw(ii)==1
        lbls{ih(ii), ie(ii)} = sprintf(...
            'h = %.2f \\lambda, \\epsilon_r = %.2f', hh, eeps_r);
    end

    % Integration of far-field.
    P_phi = phi(2)/pi * trapz((2*cos(l_eq*pi*sin(theta2D).*cos(phi2D))).^2, 2);
    G(iw(ii), ih(ii), ie(ii)) = theta(2)/pi/eta * trapz( ...
        sin(pi*w_eff*cos(theta)).^2 .* sin(theta).^3 .* P_phi ./ cos(theta).^2);
end

% Calculate radiation resistance from integrated far-field.
R_rad = 1 ./ G;

% Plot radiation resistance.
figure;
plot(w, reshape(R_rad,numel(w),[]));
axis([0.2 max(w) 0 200]);
grid on;
xlabel('Patch width [\lambda]');
ylabel('Radiation resistance at resonance [ohm]');
legend(lbls{:}, 'Location', 'southwest');

% Plot resonant lengths.
figure;
plot(w, reshape(l_res,numel(w),[]));
axis([0 max(w) 0 0.5]);
grid on;
xlabel('Patch width [\lambda]');
ylabel('Resonant length [\lambda]');
legend(lbls{:}, 'Location', 'southeast');
