% Tapered circular Gaussian aperture: radiation pattern and efficiency.
%
% Complement to Figure 7.19 in Foundations of Antennas by Per-Simon Kildal.

% By Ulf Carlberg.

% Define inputs: taper and (D/lambda)*sin(theta).
taper_dB = 0:-5:-30;
x = 0:0.01:6;

% Initialize sizes.
G0 = zeros(1, numel(taper_dB));
G = zeros(numel(x), numel(taper_dB));
lgnds = cell(1, numel(taper_dB));

% Calculate radiation patterns by integrating the radiation integral in
% Eq. (7.57).
u0 = sqrt(abs(20*log10(exp(-1)) ./ taper_dB));
u = 0:0.01:1;
for it = 1:numel(taper_dB)
    G0(1,it) = u(2) * trapz( exp(-(u/u0(it)).^2) .* u);
    for ix = 1:numel(x)
        G(ix,it) = u(2) * trapz( exp(-(u/u0(it)).^2) .* ...
            besselj(0, pi*x(ix)*u) .* u);
    end
    lgnds{it} = sprintf('Taper = %0.f dB', taper_dB(it));
end

% Normalize radiation patterns so we can plot relative levels.
G = G ./ repmat(G0, numel(x), 1);

% Plot radiation pattterns.
figure;
plot(x, 20*log10(abs(G)));
axis([min(x) max(x) -60 0]);
grid on;
xlabel('(D/\lambda) sin(\theta)');
ylabel('Relative level [dB]');
legend(lgnds);

% Redefine taper to more points.
taper_dB_eff = 0:-1:-30;
u0 = sqrt(abs(20*log10(exp(-1)) ./ taper_dB_eff));

% Initialize sizes.
G0_eff = zeros(1, numel(taper_dB_eff));
P0_eff = zeros(1, numel(taper_dB_eff));

% Calculate efficiency with the power integral.
for it = 1:numel(taper_dB_eff)
    G0_eff(it) = 2 * pi * u(2) * trapz( exp(-1*(u/u0(it)).^2) .* u);
    P0_eff(it) = 2 * pi * u(2) * trapz( exp(-2*(u/u0(it)).^2) .* u);
end
efficiency_dB = 10*log10( abs(G0_eff).^2 ./ (pi*P0_eff) );

% Plot efficiencies.
figure;
plot(-taper_dB_eff, efficiency_dB);
axis([min(-taper_dB_eff) max(-taper_dB_eff) -3 0]);
grid on;
xlabel('Aperture illumination taper [dB]');
ylabel('Aperture efficiency [dB]');
