% Surface plot of the universal radiation pattern of a H-plane sector horn.
%
% Complement to Figure 8.12 in Foundations of Antennas by Per-Simon Kildal.

% By Madeleine Kildal Schilliger

% Define inputs.
u_prime = -1:0.001:1;
v_prime = -1:0.001:1;
[uu, vv] = ndgrid(u_prime, v_prime);
min_dB = -30;
a = 5.5;    % unit [lambda]
b = 0.25;   % unit [lambda]
Ly = 6;
w = pi*2*a^2/(8*Ly);    % Equal to Phi_max in eq. 8.20 
theta = asin(sqrt(uu.^2 + vv.^2));
rho = hypot(uu, vv);

% Calculate.
t = linspace(-1, 1, 401);
dt = t(2) - t(1);
GH = zeros(size(u_prime));
for ii = 1:numel(u_prime)
    GH(ii) = dt * trapz(cos(0.5*pi*t).*exp(1j*(-w*t.^2+pi*u_prime(ii)*t*a)));   %Eq. 8.21
end
GE = zeros(size(v_prime));
for ii = 1:numel(v_prime)
    GE(ii) = dt * trapz(exp(1j*pi*v_prime(ii)*t*b));                            %Eq. 8.19, with t = 2y'/b
end
G = GH.' * GE;
G = cos(theta/2).^2 .* G;
G = G/max(max(abs(G)));

G(rho > 1) = 0;

G_dB = max(20*log10(abs(G)), min_dB);

% Plot a surf plot.
figure;
G_dB(end,end) = -100;  % hack to avoid graphic glitch when saving plot
surf(uu, vv, G_dB);
colormap(jet(-min_dB/2.5));
shading flat;
title('Radiation from a H-plane sector horn');
fs = 8;
% xlabel('$\textsf{sin($\theta$) cos($\varphi$)}$', 'Interpreter', 'latex', 'FontSize', fs);
% ylabel('$\textsf{sin($\theta$) sin($\varphi$)}$', 'Interpreter', 'latex', 'FontSize', fs);
xlabel('sin(\theta) cos(\phi)', 'Interpreter', 'tex');
ylabel('sin(\theta) sin(\phi)', 'Interpreter', 'tex');
view(0, 90);
caxis([min_dB 0]);
hb = colorbar;
colorTitleHandle = get(hb,'Title');
titleString = 'dB';
set(colorTitleHandle ,'String',titleString);
axis image;
set(gca, 'TickDir', 'out');
set(gca, 'XTickMode', 'manual');
set(gca, 'XTick', -1:0.2:1);
grid off;
box on;
hold on;
plot(u_prime, sin(acos(u_prime)),'k')
plot(u_prime,-sin(acos(u_prime)),'k')
if exist('savefigs', 'var') && savefigs
    save_fig('Fig8-12b.eps', 14, 10, 200);
    save_fig('Fig8-12b.png', 14, 10, 1500);
    colormap(flipud(gray(-min_dB/2.5)));
    save_fig('Fig8-12bgray.eps', 14, 10, 200);
    save_fig('Fig8-12bgray.png', 14, 10, 1500);
end