% Radiation patterns of a corrugated conical horn
%
% Complement to Figure 8.22 in Foundations of Antennas by Per-Simon Kildal.

% By Madeleine Kildal Schilliger.

% Define u and v.
a = 1.5;                    % a/lambda
u = -a:0.005:a;             % u = sin(theta)cos(phi)a/lambda              
v = -a:0.005:a;             % v = sin(theta)sin(phi)a/lambda
[uu, vv] = ndgrid(u,v);
rho = hypot(uu,vv);         % rho = sin(theta)*a/lambda
phi = atan(vv ./ uu);
rho_vector = linspace(0, sqrt(2)*a, 1001);

% Input parameters.
kappa01 = 2.405;            % besselj(1,x) has maximum at x=kappa01
Phi_max_deg = 90;
Phi_max = Phi_max_deg * pi / 180;
min_dB = -30;

% Calculate Eq. (8.61) and E_xp45 = 0
t = linspace(0, 1, 401);
dt = t(2) - t(1);

J0_exp_t = besselj(0, kappa01*t) .* exp(-1j*Phi_max*t.^2) .* t; 
Eco45_rho = zeros(size(rho_vector));
for ii = 1:numel(rho_vector)
    Eco45_rho(ii) =   dt * trapz(J0_exp_t.*besselj(0, 2*pi*rho_vector(ii)*t));
end
Eco45 = interp1(rho_vector, Eco45_rho, rho);
Exp45 = zeros(size(rho));

Gco = Eco45 - Exp45 .* cos(2*phi);  % Equation 8.53
Gxp = Exp45 .* sin(2*phi);          % Equation 8.53
GcoRel = Gco/max(abs(Gco(:)));
GxpRel = Gxp/max(abs(Gco(:)));

Gco = GcoRel;
Gxp = GxpRel;

Gco_dB = max(20*log10(abs(Gco)), min_dB);
Gxp_dB = max(20*log10(abs(Gxp)), min_dB);

Gco_dB(end,end) = -100;  % hack to avoid graphic glitch when saving plot
Gxp_dB(end,end) = -100;  % hack to avoid graphic glitch when saving plot

% Plot
figure;
s1 = subplot(1,2,1);
surf(uu, vv, Gco_dB);
colormap(jet(-min_dB/2.5));
shading flat;
title('CO');
% fs = 8;
% xlabel('$\textsf{u = sin($\theta$) cos($\varphi$) a/$\lambda$}$', 'Interpreter', 'latex', 'FontSize', fs);
% ylabel('$\textsf{v = sin($\theta$) sin($\varphi$) a/$\lambda$}$', 'Interpreter', 'latex', 'FontSize', fs);
xlabel('u = sin(\theta) cos(\phi) a/\lambda', 'Interpreter', 'tex');
ylabel('v = sin(\theta) sin(\phi) a/\lambda', 'Interpreter', 'tex');
view(0, 90);
caxis([min_dB 0]);
axis image;
set(gca, 'TickDir', 'out');
set(gca, 'XTickMode', 'manual');
set(gca, 'XTick', -5:5);
grid off;
box on;

s2 = subplot(1,2,2);
surf(uu, vv, Gxp_dB);
colormap(jet(-min_dB/2.5));
shading flat;
title('XP');
% fs = 8;
% xlabel('$\textsf{u = sin($\theta$) cos($\varphi$) a/$\lambda$}$', 'Interpreter', 'latex', 'FontSize', fs);
xlabel('u = sin(\theta) cos(\phi) a/\lambda', 'Interpreter', 'tex');
view(0, 90);
axis image;
set(gca, 'TickDir', 'out');
set(gca, 'XTickMode', 'manual');
set(gca, 'XTick', -5:5);
grid off;
box on;

s1Pos = get(s1,'position');
set(s1,'position',s1Pos-[0.02 0 0 0])
s2Pos = get(s2,'position');
caxis([min_dB 0]);
hb = colorbar('location','eastoutside');
set(s2,'position',s2Pos-[0.065 0 0 0]);
colorTitleHandle = get(hb,'Title');
titleString = 'dB';
set(colorTitleHandle ,'String',titleString);

ha = axes('Position',[0 0 1 0.85],'Xlim',[0 1],'Ylim',[0 1],'Box','off','Visible','off','Units','normalized', 'clipping' , 'off');
text(0.5, 1,'\bf Corrugated conical horn','HorizontalAlignment','center','VerticalAlignment', 'top')

if exist('savefigs', 'var') && savefigs
    save_fig('Fig8-22c.eps', 14, 10, 500);
    save_fig('Fig8-22c.png', 14, 10, 1500);
    colormap(flipud(pink(15)));
    save_fig('Fig8-22cgray.eps', 14, 10, 500);
    save_fig('Fig8-22cgray.png', 14, 10, 1500);
end