% Blockage and diffraction efficiency of a Cassegrain antenna.
%
% Complement to Figure 9.15a in Foundations of Antennas by Per-Simon Kildal.

% By Ulf Carlberg.

% Input.
D = logspace(1, 4, 500)';  % main reflector diameter [lambda]
A0_dB = [0.1 10 20];       % feed illumination taper [dB]
Psi_0_deg = 90;            % subtended half angle main reflector [deg]
theta_0_deg = 10;          % subtended half angle subreflector (not critical)
                           % [deg]

% Calculate optimum diameter of subreflector.
A0 = 10.^(-A0_dB/20);
[DD, AA0] = ndgrid(D, A0);
Psi_0 = Psi_0_deg*pi/180;
theta_0 = theta_0_deg*pi/180;
dd = DD .* ( cos(theta_0/2).^4 .* AA0.^2 ./ ((4*pi)^2 * sin(Psi_0) * DD) ...
            ) .^ (1/5);

% Calculate blockage and diffraction efficiency.
n = A0_dB / (-20*log10(cos(theta_0/2)));
Cb = n ./ ( 2*cot(theta_0/2)^2*(1-A0));  % blockage parameter Eq. (9.106)
Cd = cos(theta_0/2).^2 * Cb / pi;  % diffraction parameter Eq. (9.110)
Delta_cb = repmat(Cb,numel(D),1) .* (dd./DD).^2;  % Eq. (9.104)
Delta_d = (1-1j) * repmat(Cd,numel(D),1) .* sqrt(1./(dd*sin(Psi_0))) .* ...
    sqrt(1-dd./DD) .* AA0;  % Eq. (9.104)
ebd = abs(1 - Delta_cb - Delta_d).^2;

% Plot.
figure;
semilogx(D, 10*log10(ebd));
axis([min(D) max(D) -2 0]);
grid on;
title('Cassegrain antenna');
xlabel('Main reflector diameter [\lambda]');
ylabel('Blockage and diffraction efficiency [dB]');
lgnds = cell(numel(A0_dB), 1);
for ii = 1:numel(A0_dB)
    lgnds{ii} = sprintf('Feed illumination taper %.0f dB', A0_dB(ii));
end
legend(lgnds, 'Location', 'southeast');
