% Array factor of planar array.
%
% Complement to Figure 10.17b in Foundations of Antennas by Per-Simon Kildal.

% By Ulf Carlberg.

% Input.
dx = 0.7;  % array element spacing in x-direction [lambda]
dy = 1.0;  % array element spacing in y-direction [lambda]
Nx = 5;  % number of elements in x-direction
Ny = 5;  % number of elements in y-direction

% Array dimensions.
Dx = Nx*dx;  % [lambda]
Dy = Ny*dy;  % [lambda]

% Grating lobe locations.
[pp, qq] = ndgrid((1:Nx)-floor((Nx+1)/2), (1:Ny)-floor((Ny+1)/2));
xx = pp*dx;
yy = qq*dy;

% Grating lobe size.
Dx_grating = 1/Dx;
Dy_grating = 1/Dy;

% Plot.
figure;
alpha = linspace(0,2*pi,101);
plot(0.5*Dx_grating*cos(alpha), 0.5*Dy_grating*sin(alpha), 'b-');
hold on;
for ii = 1:numel(xx)
    if pp(ii)~=0 || qq(ii)~=0
        x = xx(ii) + 0.5*Dx_grating*cos(alpha);
        y = yy(ii) + 0.5*Dy_grating*sin(alpha);
        plot(x, y, 'r-');
    end
end
grid on;
axis equal;
axis(0.6*[-Nx/dx Nx/dx -Ny/dy Ny/dy]);
title('Main lobe and grating lobes');
xlabel('sin(\theta)cos(\phi)-sin(\theta_0)cos(\phi_0)');
ylabel('sin(\theta)sin(\phi)-sin(\theta_0)sin(\phi_0)');
legend('Main lobe', 'Grating lobes', 4);
if exist('savefigs', 'var') && savefigs
    save_fig('fig_10_16b.png', 14, 10, 450, 1, 0, 1);
end

%% 10.17b

% Visible region, main lobe, and grating lobes of array.
%
% Complement to Figure 10.17b in Foundations of Antennas by Per-Simon Kildal.

% By Ulf Carlberg and modified by Madeleine Kildal Schilliger

% Input.
dx = 0.7;  % array element spacing in x-direction [lambda]
dy = 1.0;  % array element spacing in y-direction [lambda]
Nx = 5;  % number of elements in x-direction
Ny = 5;  % number of elements in y-direction
theta0_deg = 30;  % main lobe direction [deg]
phi0_deg   = 45;  % main lobe direction [deg]

% Array dimensions.
Dx = Nx*dx;  % [lambda]
Dy = Ny*dy;  % [lambda]

% Grating lobe locations.
[pp, qq] = ndgrid((1:Nx)-floor((Nx+1)/2), (1:Ny)-floor((Ny+1)/2));
xx = pp*dx;
yy = qq*dy;

% Grating lobe size.
Dx_grating = 1/Dx;
Dy_grating = 1/Dy;

% Contour plot.
% Definition of the embedded element pattern.
A0_E_dB = 35;  % taper in E-plane [dB]
A0_H_dB = 10;  % taper in H-plane [dB]
theta0_E_deg = 90;  % angle of taper value in E-plane [deg]
theta0_H_deg = 90;  % angle of taper value in H-plane [deg]
plot_min_dB = -16;  % lower limit of level in surface plot [dB]

% Define cos^n pattern with y' polarization.
ne = A0_E_dB / (-20*log10(cosd(theta0_E_deg/2)));
nh = A0_H_dB / (-20*log10(cosd(theta0_H_deg/2)));
[theta, phi] = ndgrid(linspace(0,pi/2,501), linspace(-pi,pi+0.001,2001));
G_theta = cos(theta/2).^ne .* sin(phi);
GA = zeros(size(G_theta));
for ii = 1:numel(pp)
    dot_prod1 = pp(ii)*dx*sind(theta0_deg)*cosd(phi0_deg) + qq(ii)*dy*sind(theta0_deg)*sind(phi0_deg); 
    dot_prod2 = pp(ii)*dx*sin(theta).*cos(phi) + qq(ii)*dy*sin(theta).*sin(phi);
    GA = GA + exp(-2j*pi*dot_prod1)*exp(2j*pi*dot_prod2);
end
G_co_dB = 20*log10(abs(GA));
G_co_dB = G_co_dB - max(G_co_dB(:));
G_co_dB(G_co_dB<plot_min_dB) = plot_min_dB;
G_co_dB(1,1) = -100;  % avoid graphic glitch

% Plot element pattern.
figure;
surf(sin(theta).*cos(phi), sin(theta).*sin(phi), G_co_dB);
hold on;
plot(cos(phi(1,:)), sin(phi(1,:)), 'k-');
colormap(jet(-plot_min_dB));
view(0, 90);
title('Normalized 2D array pattern');
fs = 8;
%xlabel('$\textsf{u = sin($\theta$) cos($\varphi$)}$', 'Interpreter', 'latex', 'FontSize', fs);
%ylabel('$\textsf{v = sin($\theta$) sin($\varphi$)}$', 'Interpreter', 'latex', 'FontSize', fs);
xlabel('u = sin(\theta) cos(\phi)', 'Interpreter', 'tex');
ylabel('v = sin(\theta) sin(\phi)', 'Interpreter', 'tex');
grid off;
box on;
axis([-1 1 -1 1]);
axis image;
shading flat;
hb = colorbar;
caxis([plot_min_dB 0]);
colorTitleHandle = get(hb,'Title');
titleString = 'dB';
set(colorTitleHandle ,'String',titleString);

if exist('savefigs', 'var') && savefigs
    save_fig('Fig10-17b.png', 14, 10, 1000);
    colormap(flipud(pink(-plot_min_dB/2)));
    save_fig('Fig10-17bgray.png', 14, 10, 1000);
end