function [Ldip, Zdip] = impedance_dipole_microstrip(Lmin, Lmax, t, w, eta_r)
%IMPEDANCE_DIPOLE_MICROSTRIP Calculates the input impedance of thin microstrip dipoles
%
%   function [Ldip, Zdip] = impedance_dipole_microstrip(Lmin, Lmax, t, w, eta_r)
%
%   Input:
%       Lmin  - Minimum length of dipoles [lambda]
%       Lmax  - Maximum length of dipoles [lambda]
%       t     - Segment length [lambda]
%       w     - Width [lambda]
%       eta_r - Relative impedance of medium [default 1]
%
%   Output:
%       Ldip - Length of dipoles calculated [lambda]
%       Zdip - Input impedances of dipoles [ohm]

if nargin<5, eta_r = 1; end

% Constants.
k = 2*pi;  % all lengths in [lambda]
eta = 120*pi*eta_r;

% Define segments on dipoles.
Nmin = floor(Lmin/t);
Nmax = ceil(Lmax/t);
Ndipoles = Nmax - Nmin + 1;
Ldip = (Nmin:Nmax)'*t;
LL = cell(Ndipoles,1);
for ii = 1:Ndipoles
    N = Nmin + ii - 1;
    LL{ii} = t*(0:N-1)';
end

% Define current on dipoles.
cur = cell(Ndipoles,1);
for ii = 1:Ndipoles
    L = LL{ii}(end);
    cur{ii} = sin(k*(L/2-abs(LL{ii}-L/2))) / w;
end

% Calculate induced transverse electric field.
Et = zeros(Nmax,1);
tp = 0.5*t;
ww = linspace(-w/2, w/2, 401)';
uu = linspace(-t/2, t/2, 101)';
dw = ww(2) - ww(1);
du = uu(2) - uu(1);
for ii = 1:numel(Et)
    Rpp = hypot((ii-1)*t+tp/2-t/2, ww);
    Rpm = hypot((ii-1)*t+tp/2+t/2, ww);
    Rmp = hypot((ii-1)*t-tp/2-t/2, ww);
    Rmm = hypot((ii-1)*t-tp/2+t/2, ww);
    Psi_q = dw * trapz(exp(-1j*k*Rpp)./Rpp - exp(-1j*k*Rpm)./Rpm - ...
                       exp(-1j*k*Rmp)./Rmp + exp(-1j*k*Rmm)./Rmm);
    if ii==1
        % Polar integration.
        phi = linspace(0, pi/2, 1000)';
        indx = phi>atan(t/w);
        R = w ./ (2*cos(phi));
        R(indx) = t ./ (2*sin(phi(indx)));
        Psi_j = 4*phi(2)*trapz(1j/k*(exp(-1j*k*R)-1));
    else
        Psi_j = 0;
        for iu = 1:numel(uu)
            R = hypot((ii-1)*t+uu(iu), ww);
            tmp = dw * trapz(exp(-1j*k*R)./R);
            if iu==1 || iu==numel(uu), tmp = 0.5*tmp; end
            Psi_j = Psi_j + tmp;
        end
        Psi_j = du * Psi_j;
    end
    Et(ii) = 1j*eta*t*w*(k*Psi_j-Psi_q/(k*tp)) / (4*pi);
end

% For each dipole, calculate a double integral to get input impedance.
Zdip = zeros(Ndipoles,1);
for idipole = 1:Ndipoles
    Nseg = numel(cur{idipole});
    for m = 1:Nseg
        mn = abs(m-(1:Nseg)');
        Em = sum(Et(mn+1).*cur{idipole});
        Zdip(idipole) = Zdip(idipole) + Em*cur{idipole}(m);
    end
end

end
