function save_fig(filename, w_cm, h_cm, dpi)
%SAVE_FIG Save figure as a EPS or PNG file.
%
%   function save_fig(filename, w_cm, h_cm, dpi)
%
%   Saves the current figure as a PNG or EPS file, depending on filename
%   suffix.
%
%   All arguments except the filename is optional. Default is to save an
%   (approximately) 12 cm x 9 cm image file with 300 dpi.
%
%   For Matlab version 2014a and older:
%   When saving PNG files, this function first creates an EPS file and then
%   use Ghostscript to convert it to a PNG file. This is because dashed and
%   dotted lines does not look good when Matlab (version 2014a and older) saves
%   PNG files directly (with high resolution). Ghostscript must be installed,
%   and this source code may have to modified to set the location of
%   Ghostscript.

% By Ulf Carlberg.

% Default arguments.
if nargin<2 || isempty(w_cm), w_cm = 12; end
if nargin<3 || isempty(h_cm), h_cm = w_cm*9/12; end
if nargin<4 || isempty(dpi), dpi = 300; end

% Set figure size.
set(gcf, 'PaperUnits', 'centimeters');
set(gcf, 'PaperPosition', [(21-w_cm)/2 (29.7-h_cm)/2 w_cm h_cm]);

% Find filename extension.
ii = find(filename=='.', 1, 'last');
filename_ext = filename(ii:end);

% Save as EPS only and then exit?
if strcmpi(filename_ext, '.eps')
    print(gcf, '-depsc2', ['-r' num2str(dpi)], filename);
    return;
end

% Add file suffix?
if ~strcmpi(filename_ext, '.png')
    filename = [filename '.png'];
end

% Matlab version R2014b or later? Then it's easy!
if ~verLessThan('matlab','8.4.0')
    print(gcf, '-dpng', ['-r' num2str(dpi)], filename);
    return;
end

%-------------------------------------------------------------------------------

% This code is for older versions of Matlab and requires Ghostscript...
disp('Matlab version 2014b or later is recommended.');

% Paths from system environment.
ghostscript = getenv('ProgramFiles(x86)');
if isempty(ghostscript), ghostscript = getenv('ProgramFiles'); end
ghostscript = [ghostscript '\gs\gs9.09\bin\gswin32c.exe'];
tmpfile_eps = [getenv('TEMP') '\matlabplot_tmp.eps'];

% Check if Ghostscript is installed.
if isempty(dir(ghostscript))
    disp('save_fig.m expects Ghostscript to be installed in:');
    disp(['         ' ghostscript]);
    disp('         Install Ghostscipt to use save_fig.m');
    error('Ghostscipt path not found.');
end

% Save as EPS first.
print(gcf, '-depsc2', ['-r' num2str(dpi)], tmpfile_eps);

% Convert to PNG.
system(['"' ghostscript '" -dSAFER -dBATCH -dNOPAUSE -dQUIET -r' ...
    num2str(dpi) ' -sDEVICE=png16m -dTextAlphaBits=4 ' ...
    '-dGraphicsAlphaBits=4 -dEPSCrop -sOutputFile=' filename ' ' ...
    tmpfile_eps]);

%-------------------------------------------------------------------------------

end
